@@ -4,7 +4,7 @@
     created              : Sun Jan 30 12:00:15 CET 2000
     copyright            : (C) 2000 by Eric Espie
     email                : torcs@free.fr
-    version              : $Id: car.h 6254 2015-11-22 23:58:40Z kakukri $
+    version              : $Id: car.h 8697 2023-01-26 14:45:38Z torcs-ng $
 
  ***************************************************************************/
 
@@ -18,9 +18,9 @@
  ***************************************************************************/
 
 /** @file
-    		This is the car structure.
+                This is the car structure.
     @author	<a href=mailto:torcs@free.fr>Eric Espie</a>
-    @version	$Id: car.h 6254 2015-11-22 23:58:40Z kakukri $
+    @version	$Id: car.h 8697 2023-01-26 14:45:38Z torcs-ng $
     @ingroup	carstruct
     @note	Short cuts are to be used with the carElt structure.
 */
@@ -50,11 +50,11 @@
 /** Wheels Specifications */
 typedef struct
 {
-    tdble	rimRadius;	/**< Rim radius */
-    tdble	tireHeight;	/**< Tire height  */
-    tdble	tireWidth;	/**< Tire width */
-    tdble	brakeDiskRadius; /**< Brake disk radius */
-    tdble	wheelRadius;	/**< Overall wheel radius */
+    tdble	rimRadius;	        /**< Rim radius */
+    tdble	tireHeight;	        /**< Tire height  */
+    tdble	tireWidth;	        /**< Tire width */
+    tdble	brakeDiskRadius;        /**< Brake disk radius */
+    tdble	wheelRadius;	        /**< Overall wheel radius */
 } tWheelSpec;
 /* structure access short cuts */
 #define _rimRadius(i)		info.wheel[i].rimRadius 	/**< short cut to tWheelSpec#rimRadius */
@@ -70,11 +70,23 @@ typedef struct {
     tdble	exhaustPower;	/**< Power of the flames (from 1 to 3) */
 } tVisualAttributes;
 
+/** skill levels */
+enum tSkillLevel
+{
+    ARCADE,
+    SEMI_ROOKIE,
+    ROOKIE,
+    AMATEUR,
+    SEMI_PRO,
+    PRO
+};
+
 /** Static Public info */
 typedef struct {
     char	name[MAX_NAME_LEN];         /**< Driver's name */
     char	sname[MAX_NAME_LEN];        /**< Driver's short name */
     char	codename[3+1];              /**< Driver's 3-letter codename */
+    char    nationname[2+1];            /**< Driver's nationality 2-letter codename */
     char	teamname[MAX_NAME_LEN];     /**< Team name */
     char	carName[MAX_NAME_LEN];      /**< Car object name */
     char	category[MAX_NAME_LEN];     /**< Car's category */
@@ -82,26 +94,27 @@ typedef struct {
     int		startRank;                  /**< Car's starting position */
     int		driverType;                 /**< Driver type */
     int		networkplayer;              /**< Network player */
-    int		skillLevel;                 /**< Driver's skill level (0=rookie -> 3=pro) */
+    tSkillLevel	skillLevel;                 /**< Driver's skill level */
     tdble	iconColor[3];               /**< Car color in leaders board */
     t3Dd	dimension;                  /**< Car's mesures */
     t3Dd	drvPos;                     /**< Driver's position */
     t3Dd	bonnetPos;                  /**< Bonnet's position */
     tdble	tank;                       /**< Fuel tank capa */
     tdble	steerLock;                  /**< Steer lock angle */
+    tdble	steerWheelRot;              /**< Steering wheel rotation (lock to lock) */
     t3Dd	statGC;                     /**< Static pos of GC (should be the origin of car axis) */
     tWheelSpec	wheel[4];               /**< Wheels specifications */
     tVisualAttributes visualAttr;       /**< Visual attributes */
     char	masterModel[MAX_NAME_LEN];	/**< Master 3D model car name (the exact folder name) */
     char	skinName[MAX_NAME_LEN];     /**< Custom skin name, if any */
     int		skinTargets;                /**< Target objects for the custom skinning
-				   <br>The possible targets are :
-				   - RM_CAR_SKIN_TARGET_WHOLE_LIVERY
-				   - RM_CAR_SKIN_TARGET_3D_WHEELS
-				   - RM_CAR_SKIN_TARGET_INTERIOR
-				   - RM_CAR_SKIN_TARGET_BOARD
-				   - RM_CAR_SKIN_TARGET_DRIVER
-				   - RM_CAR_SKIN_TARGET_PIT_DOOR
+                                   <br>The possible targets are :
+                                   - RM_CAR_SKIN_TARGET_WHOLE_LIVERY
+                                   - RM_CAR_SKIN_TARGET_3D_WHEELS
+                                   - RM_CAR_SKIN_TARGET_INTERIOR
+                                   - RM_CAR_SKIN_TARGET_BOARD
+                                   - RM_CAR_SKIN_TARGET_DRIVER
+                                   - RM_CAR_SKIN_TARGET_PIT_DOOR
 */
 #define RM_CAR_SKIN_TARGET_WHOLE_LIVERY	 	0x00000001	/**< The whole car external livery */
 #define RM_CAR_SKIN_TARGET_3D_WHEELS        0x00000002	/**< The 3D wheels */
@@ -115,31 +128,33 @@ typedef struct {
 #define _name		info.name               /**< short cut to tInitCar#name */
 #define _sname		info.sname              /**< short cut to tInitCar#sname */
 #define _cname		info.codename           /**< short cut to tInitCar#codename */
+#define _nationname info.nationname         /**< short cut to tInitCar#nationname */
 #define _teamname	info.teamname           /**< short cut to tInitCar#teamname */
-#define _carName	info.carName			/**< short cut to tInitCar#carName */
-#define _category	info.category			/**< short cut to tInitCar#category */
-#define _driverType	info.driverType			/**< short cut to tInitCar#driverType */
-#define _networkPlayer info.networkplayer   /**< short cut to tInitCar#networkplayer*/
-#define _skillLevel	info.skillLevel			/**< short cut to tInitCar#skillLevel */
-#define _raceNumber	info.raceNumber			/**< short cut to tInitCar#raceNumber */
-#define _startRank	info.startRank			/**< short cut to tInitCar#startRank */
-#define _dimension	info.dimension			/**< short cut to tInitCar#dimension */
+#define _carName	info.carName		/**< short cut to tInitCar#carName */
+#define _category	info.category		/**< short cut to tInitCar#category */
+#define _driverType	info.driverType		/**< short cut to tInitCar#driverType */
+#define _networkPlayer  info.networkplayer      /**< short cut to tInitCar#networkplayer*/
+#define _skillLevel	info.skillLevel		/**< short cut to tInitCar#skillLevel */
+#define _raceNumber	info.raceNumber		/**< short cut to tInitCar#raceNumber */
+#define _startRank	info.startRank		/**< short cut to tInitCar#startRank */
+#define _dimension	info.dimension		/**< short cut to tInitCar#dimension */
 #define _dimension_x	info.dimension.x	/**< short cut to tInitCar#dimension.x */
 #define _dimension_y	info.dimension.y	/**< short cut to tInitCar#dimension.y */
 #define _dimension_z	info.dimension.z	/**< short cut to tInitCar#dimension.z */
-#define _drvPos_x	info.drvPos.x			/**< short cut to tInitCar#drvPos.x */
-#define _drvPos_y	info.drvPos.y			/**< short cut to tInitCar#drvPos.y */
-#define _drvPos_z	info.drvPos.z			/**< short cut to tInitCar#drvPos.z */
+#define _drvPos_x	info.drvPos.x		/**< short cut to tInitCar#drvPos.x */
+#define _drvPos_y	info.drvPos.y		/**< short cut to tInitCar#drvPos.y */
+#define _drvPos_z	info.drvPos.z		/**< short cut to tInitCar#drvPos.z */
 #define _bonnetPos_x	info.bonnetPos.x	/**< short cut to tInitCar#bonnetPos.x */
 #define _bonnetPos_y	info.bonnetPos.y	/**< short cut to tInitCar#bonnetPos.y */
 #define _bonnetPos_z	info.bonnetPos.z	/**< short cut to tInitCar#bonnetPos.z */
 #define _statGC		info.statGC             /**< short cut to tInitCar#statGC */
-#define _statGC_x	info.statGC.x			/**< short cut to tInitCar#statGC.x */
-#define _statGC_y	info.statGC.y			/**< short cut to tInitCar#statGC.y */
-#define _statGC_z	info.statGC.z			/**< short cut to tInitCar#statGC.z */
-#define _iconColor	info.iconColor			/**< short cut to tInitCar#iconColor */
+#define _statGC_x	info.statGC.x		/**< short cut to tInitCar#statGC.x */
+#define _statGC_y	info.statGC.y		/**< short cut to tInitCar#statGC.y */
+#define _statGC_z	info.statGC.z		/**< short cut to tInitCar#statGC.z */
+#define _iconColor	info.iconColor		/**< short cut to tInitCar#iconColor */
 #define _tank		info.tank               /**< short cut to tInitCar#tank */
 #define _steerLock	info.steerLock			/**< short cut to tInitCar#steerLock */
+#define _steerWheelRot	info.steerWheelRot	/**< short cut to tInitCar#steerWheelRot */
 #define _exhaustNb	info.visualAttr.exhaustNb	/**< short cut to tVisualAttributes#exhaustNb */
 #define _exhaustPos	info.visualAttr.exhaustPos	/**< short cut to tVisualAttributes#exhaustPos */
 #define _exhaustPower	info.visualAttr.exhaustPower	/**< short cut to tVisualAttributes#exhaustPower */
@@ -152,7 +167,7 @@ typedef struct {
 
 #define RM_PENALTY_DRIVETHROUGH     1
 #define RM_PENALTY_STOPANDGO        2
-#define RM_PENALTY_10SEC_STOPANDGO	3
+#define RM_PENALTY_10SEC_STOPANDGO  3
 #define RM_PENALTY_DISQUALIFIED     4
 
 /** One penalty */
@@ -176,7 +191,11 @@ typedef struct
     double*		curSplitTime;
     double		lastLapTime;
     double		curTime;
+    int                 trackPositionCount; /** number of track positions */
+    float* bestLapTimeAtTrackPosition; /** here we will store for the best laptime the current laptime for each meter of track */
+    float* currLapTimeAtTrackPosition; /** here we will store for the current laptime the current laptime for each meter of track */
     tdble		topSpeed;
+    tdble		currentMinSpeedForLap;	// Min speed on current lap, reset on start line crossing
     int			laps;
     int			bestLap;
     int			nbPitStops;
@@ -189,39 +208,49 @@ typedef struct
     tdble		distRaced;
     tdble		distFromStartLine;
     int			currentSector;
+    int			nbSectors;
+    double		trackLength;
     double		scheduledEventTime;
     tTrackOwnPit 	*pit;
     int			event;
     tCarPenaltyHead	penaltyList;	/**< List of current penalties */
+    tdble       penaltyTime;
     tdble		prevFromStartLine;
     double		wrongWayTime;
 } tCarRaceInfo;
 /* structure access */
-#define _bestLapTime		race.bestLapTime
-#define _commitBestLapTime  race.commitBestLapTime
+#define _bestLapTime	        race.bestLapTime
+#define _commitBestLapTime      race.commitBestLapTime
 #define _bestSplitTime		race.bestSplitTime
 #define _deltaBestLapTime	race.deltaBestLapTime
-#define _curLapTime         race.curLapTime
+#define _curLapTime             race.curLapTime
 #define _curSplitTime		race.curSplitTime
 #define _lastLapTime		race.lastLapTime
-#define _curTime            race.curTime
-#define _topSpeed           race.topSpeed
-#define _laps               race.laps
-#define _bestLap            race.bestLap
-#define _nbPitStops         race.nbPitStops
+#define _curTime                race.curTime
+#define _trackPositionCount     race.trackPositionCount
+#define _bestLapTimeAtTrackPosition race.bestLapTimeAtTrackPosition
+#define _currLapTimeAtTrackPosition race.currLapTimeAtTrackPosition
+#define _topSpeed               race.topSpeed
+#define _currentMinSpeedForLap	race.currentMinSpeedForLap
+#define _laps                   race.laps
+#define _bestLap                race.bestLap
+#define _nbPitStops             race.nbPitStops
 #define _remainingLaps		race.remainingLaps
-#define _pos                race.pos
+#define _pos                    race.pos
 #define _timeBehindLeader	race.timeBehindLeader
 #define _lapsBehindLeader	race.lapsBehindLeader
 #define _timeBehindPrev		race.timeBehindPrev
 #define _timeBeforeNext		race.timeBeforeNext
-#define _distRaced          race.distRaced
+#define _distRaced              race.distRaced
 #define _distFromStartLine	race.distFromStartLine
 #define _currentSector		race.currentSector
-#define _pit                race.pit
+#define _nbSectors          race.nbSectors
+#define _trackLength        race.trackLength
+#define _pit                    race.pit
 #define _scheduledEventTime	race.scheduledEventTime
-#define _event              race.event
+#define _event                  race.event
 #define _penaltyList		race.penaltyList
+#define _penaltyTime	        race.penaltyTime
 #define _prevFromStartLine	race.prevFromStartLine
 #define _wrongWayTime		race.wrongWayTime
 
@@ -229,32 +258,33 @@ typedef struct
 typedef struct {
     tDynPt	DynGC;          /**< GC data (car axis) */
     tDynPt	DynGCg;         /**< GC data (world axis) */
+    tdble	speed;		/**< total speed, sqrt(vx*vx + vy*vy + vz*vz) */
     sgMat4	posMat;         /**< position matrix */
     tTrkLocPos	trkPos;		/**< current track position. The segment is the track segment (not sides)*/
     int		state;	    	/**< state of the car.
-				   <br>The states are:
-				   - RM_CAR_STATE_FINISH
-				   - RM_CAR_STATE_PIT
-				   - RM_CAR_STATE_DNF
-				   - RM_CAR_STATE_PULLUP
-				   - RM_CAR_STATE_PULLSIDE
-				   - RM_CAR_STATE_PULLDN
-				   - RM_CAR_STATE_OUT
-				   - RM_CAR_STATE_NO_SIMU
-				   - RM_CAR_STATE_BROKEN
-				   - RM_CAR_STATE_OUTOFGAS
-				   - RM_CAR_STATE_ELIMINATED
-				   - RM_CAR_STATE_ENDRACE_CALLED
-				   - RM_CAR_STATE_SIMU_NO_MOVE
+                                   <br>The states are:
+                                   - RM_CAR_STATE_FINISH
+                                   - RM_CAR_STATE_PIT
+                                   - RM_CAR_STATE_DNF
+                                   - RM_CAR_STATE_PULLUP
+                                   - RM_CAR_STATE_PULLSIDE
+                                   - RM_CAR_STATE_PULLDN
+                                   - RM_CAR_STATE_OUT
+                                   - RM_CAR_STATE_NO_SIMU
+                                   - RM_CAR_STATE_BROKEN
+                                   - RM_CAR_STATE_OUTOFGAS
+                                   - RM_CAR_STATE_ELIMINATED
+                                   - RM_CAR_STATE_ENDRACE_CALLED
+                                   - RM_CAR_STATE_SIMU_NO_MOVE
 */
 #define RM_CAR_STATE_FINISH	 	0x00000100				/**< Car having passed the finish line */
 #define RM_CAR_STATE_PIT	 	0x00000001				/**< Car currently stopped in pits */
 #define RM_CAR_STATE_DNF	 	0x00000002				/**< Car did not finish */
 #define RM_CAR_STATE_PULLUP	 	0x00000004				/**< Car pulled out in the air */
-#define RM_CAR_STATE_PULLSIDE	0x00000008				/**< Car pulled out in the air */
+#define RM_CAR_STATE_PULLSIDE	        0x00000008				/**< Car pulled out in the air */
 #define RM_CAR_STATE_PULLDN	 	0x00000010				/**< Car pulled out in the air */
 #define RM_CAR_STATE_OUT		(RM_CAR_STATE_DNF | RM_CAR_STATE_FINISH)/**< Car out of race */
-#define RM_CAR_STATE_NO_SIMU	0x000000FF				/**< Do not simulate the car */
+#define RM_CAR_STATE_NO_SIMU	        0x000000FF				/**< Do not simulate the car */
 #define RM_CAR_STATE_BROKEN	 	0x00000200				/**< Engine no more working */
 #define RM_CAR_STATE_OUTOFGAS	 	0x00000400				/**< Out of Gas */
 #define RM_CAR_STATE_ELIMINATED	 	0x00000800				/**< Eliminated due to rules infringement */
@@ -292,24 +322,25 @@ typedef struct {
 
 /** Dynamic wheel information */
 typedef struct {
-    tPosd	relPos;				/**< position relative to GC */
-    tdble	spinVel;			/**< spin velocity rad/s */
-    tdble	brakeTemp;			/**< brake temperature from 0 (cool) to 1.0 (hot) */
-    int		state;				/**< wheel state */
-    tTrackSeg	*seg;			/**< Track segment where the wheel is */
-    tdble rollRes;              /**< rolling resistance, useful for sound */
-	tdble   temp_in, temp_mid, temp_out;
-	tdble   condition;      	/**< tire condition, between 0 and 1 */
-    tdble   treadDepth;     	/**< tread depth, between 0 and 1 */
-    tdble   critTreadDepth; 	/**< critical tread depth, when grip falls off suddenly, between 0 and treadDepth */
-	tdble slipNorm; 			/* normalized slip, the variable of Magic Formula */
-	tdble slipOpt;				/* the value of slipNorm giving maximal grip */
-	tdble slipSide;
-	tdble slipAccel;
-	tdble Fx;
-	tdble Fy;
-	tdble Fz;
-	tdble effectiveMu;
+  tPosd	        relPos;			                /**< position relative to GC */
+  tdble	        spinVel;		                /**< spin velocity rad/s */
+  tdble	        brakeTemp;		                /**< brake temperature from 0 (cool) to 1.0 (hot) */
+  int		    state;			                    /**< wheel state */
+  tTrackSeg	    *seg;			                    /**< Track segment where the wheel is */
+  tdble         rollRes;                        /**< rolling resistance, useful for sound */
+  tdble         temp_in, temp_mid, temp_out;    /**< tire temperature inside, middle and outside of tread */
+  tdble         temp_opt;                       /**< optimal tire temperature */
+  tdble         condition;      	            /**< tire condition, between 0 and 1 */
+  tdble         treadDepth;     	            /**< tread depth, between 0 and 1 */
+  tdble         critTreadDepth; 	            /**< critical tread depth, when grip falls off suddenly, between 0 and treadDepth */
+  tdble         slipNorm; 		                /**< normalized slip, the variable of Magic Formula */
+  tdble         slipOpt;		                /**< the value of slipNorm giving maximal grip */
+  tdble         slipSide;
+  tdble         slipAccel;
+  tdble         Fx;
+  tdble         Fy;
+  tdble         Fz;
+  tdble         effectiveMu;
 } tWheelState;
 /* structure access */
 #define _ride(i)                priv.wheel[i].relPos.z
@@ -326,6 +357,7 @@ typedef struct {
 #define _tyreT_in(i)            priv.wheel[i].temp_in
 #define _tyreT_mid(i)           priv.wheel[i].temp_mid
 #define _tyreT_out(i)           priv.wheel[i].temp_out
+#define _tyreT_opt(i)           priv.wheel[i].temp_opt
 #define _tyreCondition(i)       priv.wheel[i].condition
 #define _tyreTreadDepth(i)      priv.wheel[i].treadDepth
 #define _tyreCritTreadDepth(i)  priv.wheel[i].critTreadDepth
@@ -335,9 +367,9 @@ typedef struct {
 
 typedef struct tCollisionState_
 {
-	int collision_count;
-	sgVec3 pos;
-	sgVec3 force;
+  int    collision_count;
+  sgVec3 pos;
+  sgVec3 force;
 } tCollisionState;
 /* structure access */
 #define _collCount	priv.collision_state.collision_count
@@ -347,12 +379,12 @@ typedef struct tCollisionState_
 typedef struct MemoryPoolItem tMemoryPoolItem;
 typedef tMemoryPoolItem* tMemoryPool;
 
-typedef struct MemPoolCar 
+typedef struct MemPoolCar
 {
-	tMemoryPool newTrack;
-	tMemoryPool newRace;
-	tMemoryPool endRace;
-	tMemoryPool shutdown;
+    tMemoryPool newTrack;
+    tMemoryPool newRace;
+    tMemoryPool endRace;
+    tMemoryPool shutdown;
 } tMemPoolCar;
 
 /* structrure to store one parameter of car setup */
@@ -408,8 +440,8 @@ typedef struct
     int		gear;                   /**< current gear */
     int		gearNext;               /**< next gear while shiting */
     tdble	fuel;                   /**< remaining fuel (liters) */
-	tdble   fuel_consumption_total; // l
-	tdble   fuel_consumption_instant; // l/100km (>100 means infinity)
+    tdble   fuel_consumption_total; /**< l */
+    tdble   fuel_consumption_instant; /**< l/100km (>100 means infinity) */
     tdble	enginerpm;
     tdble	enginerpmRedLine;
     tdble	enginerpmMax;
@@ -424,20 +456,23 @@ typedef struct
     tdble	reaction[4];            /**< reaction on wheels */
     int		collision;
     int		simcollision;           /**< For rules etc. reflects the collision state from simu */
-	float   smoke;
+    float   smoke;
     t3Dd	normal;
     t3Dd	collpos;                /**< Collision position, useful for sound ; Simu V2 only */
     int		dammage;
     int		debug;
-	tCollisionState collision_state; /**< collision state ; Simu V3 only  */
+    tdble   localairTemperature;     /** < air temperature in race */
+    tdble   localairPressure;        /**< Environment pressure at cars location */
+    tCollisionState collision_state; /**< collision state ; Simu V3 only  */
     tMemPoolCar	memoryPool;
     tdble       driveSkill;          /**< Skill level for robots: 0.0 means as fast as possible; 10.0 means at a slower speed so players can easier win */
-    tdble       steerTq;            /**< torque on steering wheel for force feedback */
+    tdble       steerTqCenter;       /**< torques on steering wheel for force feedback, this is the centering torque, linear with steering angle */
+    tdble	steerTqAlign;		     /**< force feedback torque: tire aligning torque from magic formula */
     tDashboardItem dashboardInstant[NR_DI_INSTANT];
-    int		dashboardInstantNb;	/* number and list of immediately changing items in dashboard */
+    int		dashboardInstantNb;	     /**< number and list of immediately changing items in dashboard */
     tDashboardItem dashboardRequest[NR_DI_REQUEST];
-    int		dashboardRequestNb;	/* number and list of items requested to change during next pit stop */
-    int		dashboardActiveItem;	/* active item in dashboard, 0 .. dashboardInstantNb+dashboardRequestNb-1 */
+    int		dashboardRequestNb;	     /**< number and list of items requested to change during next pit stop */
+    int		dashboardActiveItem;	 /**< active item in dashboard, 0 .. dashboardInstantNb+dashboardRequestNb-1 */
 } tPrivCar;
 /* structure access */
 #define _fuelTotal      priv.fuel_consumption_total
@@ -461,6 +496,8 @@ typedef struct
 #define _gear           priv.gear
 #define _gearNext       priv.gearNext
 #define _debug          priv.debug
+#define _airtemp        priv.localairTemperature
+#define _airPressure    priv.localairPressure
 #define _skid           priv.skid
 #define _reaction       priv.reaction
 #define _dammage        priv.dammage
@@ -470,7 +507,9 @@ typedef struct
 #define _smoke          priv.smoke
 #define _normal         priv.normal
 #define _coll2Pos       priv.collpos
-#define _steerTq        priv.steerTq
+
+#define _steerTqCenter		priv.steerTqCenter
+#define _steerTqAlign		priv.steerTqAlign
 
 #define _newTrackMemPool	priv.memoryPool.newTrack
 #define _newRaceMemPool		priv.memoryPool.newRace
@@ -492,40 +531,40 @@ typedef struct
     tdble	brakeCmd;           /**< Brake command       [ 0.0, 1.0] */
     tdble	clutchCmd;          /**< Clutch command      [ 0.0, 1.0] */
 
-	// New commands for single wheel braking
-	tdble	brakeFrontLeftCmd;  /**< Brake front left command  [0.0, 1.0] 0: no brake; 1: max brake */
+    // New commands for single wheel braking
+    tdble	brakeFrontLeftCmd;  /**< Brake front left command  [0.0, 1.0] 0: no brake; 1: max brake */
     tdble	brakeFrontRightCmd; /**< Brake front right command [0.0, 1.0] 0: no brake; 1: max brake */
     tdble	brakeRearLeftCmd;   /**< Brake rear left command   [0.0, 1.0] 0: no brake; 1: max brake */
     tdble	brakeRearRightCmd;  /**< Brake rear right command  [0.0, 1.0] 0: no brake; 1: max brake */
 
-	// New commands for variable wings (incl. airbrake)
+    // New commands for variable wings (incl. airbrake)
     tdble	wingFrontCmd;       /**< Wing angle of attack front [0, PI/4: angle in rad] */
     tdble	wingRearCmd;        /**< Wing angle of attack rear  [0, PI/4: angle in rad] */
     tdble	reserved1;          /**< reserved for future use */
     tdble	reserved2;          /**< reserved for future use */
 
-	int		gear;  	            /**< [-1,MAX_GEARS - 2] for gear selection */
+    int		gear;  	            /**< [-1,MAX_GEARS - 2] for gear selection */
     int		raceCmd;            /**< command issued by the driver */
 #define RM_CMD_NONE		0	    /**< No race command */
 #define RM_CMD_PIT_ASKED	1	/**< Race command: Pit asked */
 #define RM_CMD_MAX_MSG_SIZE 32
 
-	int		lightCmd;           /**< Lights command */
+    int	lightCmd;           /**< Lights command */
 #define RM_LIGHT_HEAD1		0x00000001	/**< head light 1 */
 #define RM_LIGHT_HEAD2		0x00000002	/**< head light 2 */
 
-	int		ebrakeCmd;          /**< Parking/Emergency Brake command */
+    int	ebrakeCmd;          /**< Parking/Emergency Brake command */
 
-	int	    wingControlMode;        /**< 0: No wing control; 1 fixed wing angles; 2 variable wing angles */
-	int	    singleWheelBrakeMode;   /**< 0: No single wheel brake; 1 single wheel brake mode */
-	int	    switch3;                /**< reserved for future use */
-	int	    telemetryMode;          /**< Define mode for telemetry output: 0: no output; 1: full output; ... see car.cpp SimCarUpdate2 */
+    int	wingControlMode;        /**< 0: No wing control; 1 fixed wing angles; 2 variable wing angles */
+    int	singleWheelBrakeMode;   /**< 0: No single wheel brake; 1 single wheel brake mode */
+    int	switch3;                /**< reserved for future use */
+    int	telemetryMode;          /**< Define mode for telemetry output: 0: no output; 1: full output; ... see car.cpp SimCarUpdate2 */
 
-	char	msg[4][RM_CMD_MAX_MSG_SIZE];     /**< 4 lines of 31 characters : 0-1 from car, 2-3 from race engine */
+    char	msg[4][RM_CMD_MAX_MSG_SIZE];     /**< 4 lines of 31 characters : 0-1 from car, 2-3 from race engine */
 #define RM_MSG_LEN	31
 
-	float	msgColor[4]; /**< RGBA of text */
-	tDashboardItem *setupChangeCmd;	/* setup item changed in dashboard */
+    float	msgColor[4]; /**< RGBA of text */
+    tDashboardItem *setupChangeCmd;	/* setup item changed in dashboard */
 } tCarCtrl;
 #define _steerCmd	ctrl.steer
 #define _accelCmd	ctrl.accelCmd
@@ -561,11 +600,11 @@ typedef struct
     tCarSetupItem wingAngle[2];
     tCarSetupItem revsLimiter;
     tCarSetupItem gearRatio[MAX_GEARS];
-    int differentialType[3];
+    int           differentialType[3];
     tCarSetupItem differentialRatio[3];
     tCarSetupItem differentialMinTqBias[3], differentialMaxTqBias[3], differentialViscosity[3];
     tCarSetupItem differentialLockingTq[3], differentialMaxSlipBias[3], differentialCoastMaxSlipBias[3];
-    tCarSetupItem steerLock;
+    tCarSetupItem steerLock, steerWheelRot;
     tCarSetupItem brakeRepartition, brakePressure;
     tCarSetupItem rideHeight[4], toe[4], camber[4];
     tCarSetupItem tirePressure[4], tireOpLoad[4]; //note: pressure is unused now
@@ -589,6 +628,9 @@ typedef struct CarPitCmd
 #define RM_PIT_STOPANDGO	1
     int			stopType;
     bool		setupChanged; /* TRUE if setup has changed during the pit stop */
+
+    enum TireChange { NONE = 0, ALL = 1};
+    TireChange tireChange;
 } tCarPitCmd;
 /* structure access */
 #define _pitFuel	pitcmd.fuel
@@ -677,38 +719,44 @@ typedef struct CarElt
 #define PRM_TANK			"fuel tank"
 #define PRM_FUEL			"initial fuel"
 #define PRM_CENTR			"mass repartition coefficient"
+#define PRM_ROLLROTINERTIA		"roll rotational inertia"
+#define PRM_PITCHROTINERTIA		"pitch rotational inertia"
+#define PRM_YAWROTINERTIA		"yaw rotational inertia"
 #define PRM_INERTIA			"inertia"
 #define PRM_EFFICIENCY		"efficiency"
 #define PRM_TYPE			"type"
 #define PRM_SIZE			"size"
 
 /* Tires */
-#define PRM_MU					"mu"
-#define PRM_RIMDIAM				"rim diameter"
+#define PRM_MU				"mu"
+#define PRM_RIMDIAM			"rim diameter"
 #define PRM_TIREWIDTH			"tire width"
 #define PRM_TIREHEIGHT			"tire height"
 #define PRM_TIRERATIO			"tire height-width ratio"
 #define PRM_RIDEHEIGHT			"ride height"
 #define PRM_ROLLINGRESIST		"rolling resistance"
-#define PRM_TOE					"toe"
-#define PRM_CAMBER				"camber"
-#define PRM_CA					"stiffness"
-#define PRM_RFACTOR				"dynamic friction"
-#define PRM_EFACTOR				"elasticity factor"
+#define PRM_TOE				"toe"
+#define PRM_CAMBER			"camber"
+#define PRM_CA				"stiffness"
+#define PRM_RFACTOR			"dynamic friction"
+#define PRM_EFACTOR			"elasticity factor"
 #define PRM_PRESSURE			"pressure"
 #define PRM_LOADFMAX			"load factor max"
 #define PRM_LOADFMIN			"load factor min"
-#define PRM_OPLOAD				"operating load"
-#define PRM_OPTTEMP				"optimal temperature"
+#define PRM_OPLOAD			"operating load"
+#define PRM_ALIGNTQFACTOR		"aligning torque factor"
+#define PRM_OPTTEMP			"optimal temperature"
 #define PRM_INITTEMP			"initial temperature"
 #define PRM_COLDMUFACTOR		"cold mu factor"
 #define PRM_HEATINGMULT			"heating multiplier"
 #define PRM_AIRCOOLINGMULT		"air cooling multiplier"
-#define PRM_SPEEDCOOLINGMULT	"speed cooling multiplier"
+#define PRM_SPEEDCOOLINGMULT	        "speed cooling multiplier"
 #define PRM_WEARRATE			"wear rate multiplier"
-#define PRM_FALLOFFTREADDEPTH	"falloff tread depth"
+#define PRM_FALLOFFTREADDEPTH	        "falloff tread depth"
 #define PRM_FALLOFFGRIPMULT		"falloff grip multiplier"
-#define PRM_REMAININGGRIPMULT	"remaining grip multiplier"
+#define PRM_REMAININGGRIPMULT	        "remaining grip multiplier"
+#define PRM_IDEALTEMP	                "ideal temperature"
+#define PRM_WEAR		        "wear"
 
 #define PRM_SPR				"spring"
 #define PRM_SUSPCOURSE		"suspension course"
@@ -727,6 +775,7 @@ typedef struct CarElt
 
 #define PRM_STEERLOCK		"steer lock"
 #define PRM_STEERSPD		"max steer speed"
+#define PRM_STEERROT		"steering wheel rotation"
 
 #define PRM_BRKDIAM			"disk diameter"
 #define PRM_BRKAREA			"piston area"
@@ -803,12 +852,29 @@ typedef struct CarElt
 #define PRM_SW_MODEL        "model"
 #define PRM_SW_MODELHR      "hi res model"
 #define PRM_SW_ANGLE        "angle"
-#define PRM_SW_MOVT         "movt"
 
 #define LST_DRIVER          "Driver"
 #define PRM_DRIVERSTEER     "steer"
 #define PRM_DRIVERMODEL     "driver"
 
+#define SECT_INSTRUMENTS    "Instruments"
+#define PRM_RPM_A_MODEL     "rpm analogic model"
+#define PRM_SPEED_A_MODEL   "speed analogic model"
+#define PRM_FUEL_A_MODEL    "fuel analogic model"
+#define PRM_ABS_A_MODEL     "abs analogic model"
+#define PRM_TCL_A_MODEL     "tcl analogic model"
+#define PRM_WATER_A_MODEL   "water analogique model"
+#define PRM_OIL_A_MODEL     "oil analogic model"
+
+#define PRM_RPM_D_MODEL     "rpm digital model"
+#define PRM_SPEED_D_MODEL   "speed digital model"
+#define PRM_SPD_D_MODEL     "spd digital model"
+#define PRM_FUEL_D_MODEL    "fuel digital model"
+#define PRM_ABS_D_MODEL     "abs digital model"
+#define PRM_TCL_D_MODEL     "tcl digital model"
+#define PRM_WATER_D_MODEL   "water digital model"
+#define PRM_OIL_D_MODEL     "oil digital model"
+
 #define LST_REARWING        "RearWing"
 #define PRM_REARWINGANGLE   "angle"
 #define PRM_REARWINGMODEL   "rearwing"
@@ -838,15 +904,40 @@ typedef struct CarElt
 #define VAL_DIFF_15WAY_LSD	"1.5 WAY LSD"
 #define VAL_DIFF_ELECTRONIC_LSD	"ELECTRONIC LSD"
 
+typedef enum
+{
+    TRANS_RWD,
+    TRANS_FWD,
+    TRANS_4WD,
+} tDriveType;
+
 #define VAL_TRANS_RWD		"RWD"
 #define VAL_TRANS_FWD		"FWD"
 #define VAL_TRANS_4WD		"4WD"
 
+typedef enum
+{
+    ENGSHAPE_V,
+    ENGSHAPE_L,
+    ENGSHAPE_H,
+    ENGSHAPE_W
+
+} tEngineShape;
+
 #define VAL_ENGSHAPE_V		"v"
 #define VAL_ENGSHAPE_L		"l"
 #define VAL_ENGSHAPE_H		"h"
 #define VAL_ENGSHAPE_W		"w"
 
+typedef enum
+{
+    ENGPOS_FRONT,
+    ENGPOS_FRONTMID,
+    ENGPOS_MID,
+    ENGPOS_REARMID,
+    ENGPOS_REAR,
+} tEnginePosition;
+
 #define VAL_ENGPOS_FRONT		"front"
 #define VAL_ENGPOS_FRONTMID		"front-mid"
 #define VAL_ENGPOS_MID          "mid"
@@ -915,10 +1006,10 @@ typedef struct CarElt
 
 // Collision constants.
 #define SEM_COLLISION			0x01
-#define SEM_COLLISION_XYSCENE	0x02
+#define SEM_COLLISION_XYSCENE	        0x02
 #define SEM_COLLISION_CAR		0x04
 #define SEM_COLLISION_Z			0x08
-#define SEM_COLLISION_Z_CRASH	0x10
+#define SEM_COLLISION_Z_CRASH	        0x10
 
 /* features */
 #define PRM_AEROTOCG            "shifting aero coordinates"
@@ -931,6 +1022,11 @@ typedef struct CarElt
 #define PRM_ABSINSIMU           "enable abs"
 #define PRM_ESPINSIMU           "enable esp"
 #define PRM_LIMITEDGROUNDEFFECT	"enable cliftlimit"
+#define PRM_TCLBRAKESCALE       "tcl brake scale"
+#define PRM_TCLACCELSCALE       "tcl accel scale"
+#define PRM_TCLSLIPSCALE        "tcl slip scale"
+#define PRM_ABSBRAKESCALE       "abs brake scale"
+#define PRM_ABSSLIPSCALE        "abs slip scale"
 #define VAL_YES                 "yes"
 #define VAL_NO                  "no"
 #define FEAT_AEROTOCG               0x01        //shift aero coordinates with the CG
@@ -942,6 +1038,6 @@ typedef struct CarElt
 #define FEAT_TCLINSIMU              0x40        //TCL simulation with 500 Hz at single wheels and engine
 #define FEAT_ABSINSIMU              0x80        //ABS simulation with 500 Hz at single wheels
 #define FEAT_ESPINSIMU              0x100       //ESP simulation with 500 Hz at single wheels
-#define FEAT_LIMITEDGROUNDEFFECT	0x200       //Limit for Clift is enabled
+#define FEAT_LIMITEDGROUNDEFFECT    0x200       //Limit for Clift is enabled
 
-#endif /* __CARV1_H__ */ 
+#endif /* __CARV1_H__ */
