@@ -3,7 +3,7 @@
     file                 : glfeatures.h
     created              : Wed Jun 1 14:56:31 CET 2005
     copyright            : (C) 2005 by Bernhard Wymann
-    version              : $Id: glfeatures.h 5180 2013-02-23 13:56:49Z torcs-ng $
+    version              : $Id: glfeatures.h 8719 2023-02-01 05:16:39Z beaglejoe $
 
 ***************************************************************************/
 
@@ -28,7 +28,7 @@
 #ifdef __APPLE__
 #  include <OpenGL/gl.h>
 #  include <OpenGL/glu.h>
-#  include <OpenGL/glext.h>
+//#  include <OpenGL/glext.h>
 #else //__APPLE__
 #  include <GL/gl.h>
 #  include <GL/glu.h>
@@ -129,8 +129,8 @@ class TGFCLIENT_API GfglFeatures
 	GfglFeatures();
 
 	// Update supported OpenGL features according to the given frame buffer specs.
-    bool detectBestSupport(int& nWidth, int& nHeight, int& nDepth,
-                           bool& bAlpha, bool& bBump, bool& bStereo, bool& bFullScreen, int& nAniFilt);
+	bool detectBestSupportSDL2(int& nWidth, int& nHeight, int& nDepth,
+                           bool& bAlpha, bool& bFullScreen, bool& bBump, bool& bStereo, int& nAniFilt);
 
 	bool loadSupport(int &nWidth, int &nHeight, int &nDepth,
                      bool &bAlpha, bool &bFullScreen, bool &bBump, bool &bStereo, int &nAniFilt, void* hparmConfig = 0);
