@@ -4,7 +4,7 @@
     created              : Sun Jan 30 22:58:45 CET 2000
     copyright            : (C) 2000 by Eric Espie
     email                : torcs@free.fr
-    version              : $Id: graphic.h 5513 2013-06-12 22:29:08Z torcs-ng $
+    version              : $Id: graphic.h 8130 2022-04-28 04:00:49Z beaglejoe $
 
  ***************************************************************************/
 
@@ -16,8 +16,8 @@
  *   (at your option) any later version.                                   *
  *                                                                         *
  ***************************************************************************/
- 
- 
+
+
 #ifndef _GRAPHV1_H_
 #define _GRAPHV1_H_
 
@@ -40,6 +40,7 @@
 #define GR_ATT_BOARDWIDTH	"board width"
 #define GR_ATT_SPEEDORISE	"speedometer vertical position"
 #define GR_ATT_NBLEADER		"Max leaders entries"
+#define GR_ATT_DASHBOARD    "dashboard"
 
 #define GR_SCT_TVDIR		"TV Director View"
 #define GR_ATT_CHGCAMINT	"change camera interval"
@@ -71,16 +72,25 @@
 #define GR_ATT_CLOUDLAYER	"cloudlayer"
 #define GR_ATT_AGR_FOREST	"forest LOD"
 #define GR_ATT_AGR_TREE		"tree LOD"
-#define GR_ATT_AGR_PARKING	"parking LOD"
+#define GR_ATT_AGR_QUALITY	"shadow quality"
 //#define GR_ATT_DYNAMICWEATHER	"dynamic weather"
 
 #define GR_ATT_SHADOW_TYPE  "shadow type"
-#define GR_ATT_SHADOW_NONE  "shadow none"
-#define GR_ATT_SHADOW_SM    "shadow map"
-#define GR_ATT_SHADOW_SSM   "soft shadow map"
-#define GR_ATT_SHADOW_PSSM  "paralele shadow map"
-#define GR_ATT_SHADOW_LSPM  "light shadow map"
+#define GR_ATT_SHADOW_NONE  "shadow static"
+#define GR_ATT_SHADOW_SM    "shadowmap"
+#define GR_ATT_SHADOW_SSM   "soft shadowmap"
+#define GR_ATT_SHADOW_PSSM  "parallel shadowmap"
+#define GR_ATT_SHADOW_LSPM  "light shadowmap"
+#define GR_ATT_SHADOW_VDSM  "view dependent shadowmap"
+
 #define GR_ATT_SHADOW_SIZE  "texture shadow size"
+#define GR_ATT_SHADOW_512   "512"
+#define GR_ATT_SHADOW_1024  "1024"
+#define GR_ATT_SHADOW_2048  "2048"
+#define GR_ATT_SHADOW_4096  "4096"
+#define GR_ATT_SHADOW_8192  "8192"
+
+#define GR_ATT_SHADERS      "shaders"
 
 #define GR_ATT_ENV_TYPE     "texture environment"
 #define GR_ATT_AGR_NULL		"none"
@@ -88,6 +98,7 @@
 #define GR_ATT_AGR_MEDIUM	"medium"
 #define GR_ATT_AGR_FULL		"full"
 #define GR_ATT_AGR_HIGH		"high"
+#define GR_ATT_AGR_ULTRA	"ultra"
 #define GR_ATT_ENV_SIZE     "environment size"
 
 #define GR_ATT_NB_SCREENS	"number of screens"
@@ -105,7 +116,7 @@
 #define GR_ATT_REAR_LEVEL2      "Rear Level Group 2"
 #define GR_ATT_REAR_LEVEL1      "Rear Level Group 1"
 
-#define GR_ATT_FRONT_MAP1       "Front Level Map 1" 
+#define GR_ATT_FRONT_MAP1       "Front Level Map 1"
 #define GR_ATT_FRONT_MAP2       "Front Level Map 2"
 #define GR_ATT_FRONT_MAP3       "Front Level Map 3"
 #define GR_ATT_REAR_MAP1        "Rear Level Map 1"
@@ -114,6 +125,7 @@
 
 #define GR_SCT_MONITOR          "Monitor"
 #define GR_ATT_MONITOR          "monitor type"
+#define GR_VAL_MONITOR_21BY9    "21:9"
 #define GR_VAL_MONITOR_16BY9	"16:9"
 #define GR_VAL_MONITOR_4BY3		"4:3"
 #define GR_VAL_MONITOR_NONE		"none"
