@@ -4,7 +4,7 @@
     created              : Mon Jul 28 19:48:14 CEST 2012
     copyright            : (C) 2012 by Gaëtan André
     web                  : http://www.speed-dreams.org
-    version              : $Id: isoundengine.h 5095 2013-01-12 17:57:47Z pouillot $
+    version              : $Id: isoundengine.h 8380 2022-08-08 01:15:06Z beaglejoe $
  ***************************************************************************/
 
 /***************************************************************************
@@ -30,6 +30,8 @@ public:
     virtual void shutdown() = 0;
     virtual void refresh(struct Situation *s, Camera *camera) = 0;
     virtual void mute(bool bOn = true) = 0;
+    virtual float getVolume() const = 0;
+    virtual void setVolume(float g) = 0;
 };
  
  #endif // __ISOUNDENGINE__H__
