@@ -4,7 +4,7 @@
     created     : Fri Jul 8 15:19:34 CET 2005
     copyright   : (C) 2005 Bernhard Wymann
     email       : berniw@bluewin.ch
-    version     : $Id: portability.h 5308 2013-03-12 14:55:59Z beaglejoe $
+    version     : $Id: portability.h 8415 2022-08-19 19:16:13Z iobyte $
 
  ***************************************************************************/
 
@@ -26,6 +26,12 @@
 #define _USE_MATH_DEFINES
 #include <cmath>
 
+#if ( __GNUC__ >= 5 || _MSVC_VER >= 1910)
+#define SD_FALLTHROUGH [[fallthrough]];
+#else
+#define SD_FALLTHROUGH // [[fallthrough]]
+#endif
+
 #ifdef _MSC_VER
 #include <direct.h>
 #include <process.h>
@@ -94,19 +100,51 @@ PORTABILITY_API char *strtok_r(char *str, const char *delim, char **nextp);
 // * MSVC 2003 : 1300 <= _MSC_VER < 1400
 // * MSVC 2005 : 1400 <= _MSC_VER < 1500
 // * MSVC 2008 : 1500 <= _MSC_VER < 1600
-// * MSVC 2010 : 1600 <= _MSC_VER
+// * MSVC 2010 : 1600 <= _MSC_VER < 1700
+// * MSVC 2012 : 1700 <= _MSC_VER < 1800
+// * MSVC 2013 : 1800 <= _MSC_VER < 1900
+// * MSVC 2015 : 1900 <= _MSC_VER < 1910
+// * MSVC 2017 : 1910 <= _MSC_VER < 1920
+// * MSVC 2019 : 1920 <= _MSC_VER
 #ifdef _MSC_VER
 
+#if _MSC_VER < 1900
 #define isnan _isnan
+#define isfinite _finite
+#endif
+#if _MSC_VER < 1800
+#define copysign _copysign
 
-// Ticket #663 - MSVC implementation of snprintf is not safe
-// We provide our own version of the function,
-// that ensures 0 ending for the string.
-PORTABILITY_API int SD_snprintf(char *str, size_t size, const char *format, ...);
+#include <string>
+
+namespace std
+{
+    inline string to_string(size_t value)
+    {
+        return to_string(static_cast<unsigned long long>(value));
+    }
+
+    inline string to_string(int value)
+    {
+        return to_string(static_cast<long long>(value));
+    }
+}
+#endif
 
 #include <cstdarg>
 #include <cstdio>
+
+#if _MSC_VER < 1700
+#define nullptr NULL
+#endif
+
+// Ticket #663 - MSVC implementation of snprintf prior to VS 2015 is not safe
+// We provide our own version of the function,
+// that ensures 0 ending for the string.
+#if _MSC_VER < 1900
+PORTABILITY_API int SD_snprintf(char *str, size_t size, const char *format, ...);
 #define snprintf SD_snprintf
+#endif // _MSC_VER < 1900
 
 #define access _access
 
