@@ -4,7 +4,7 @@
     created              : Sun Jan 30 22:59:17 CET 2000
     copyright            : (C) 2000,2002 by Eric Espie
     email                : torcs@free.fr
-    version              : $Id: raceman.h 6142 2015-09-24 15:16:10Z torcs-ng $
+    version              : $Id: raceman.h 7889 2022-01-09 01:58:57Z torcs-ng $
 
  ***************************************************************************/
 
@@ -16,14 +16,14 @@
  *   (at your option) any later version.                                   *
  *                                                                         *
  ***************************************************************************/
- 
+
 /** @file
-    		This is the race information structures.
+                This is the race information structures.
     @author	<a href=mailto:torcs@free.fr>Eric Espie</a>
-    @version	$Id: raceman.h 6142 2015-09-24 15:16:10Z torcs-ng $
+    @version	$Id: raceman.h 7889 2022-01-09 01:58:57Z torcs-ng $
     @ingroup	raceinfo
 */
- 
+
 #ifndef _RACEMANV1_H_
 #define _RACEMANV1_H_
 
@@ -50,11 +50,12 @@ typedef int (*tfRmRunState) (struct RmInfo *);
 #define RM_NEXT_EVENT		0x00000400
 
 #define RM_QUIT			0x10000000
-#define RM_ERROR			0x2000000
+#define RM_ERROR	        0x2000000
 
 
 #define RCM_MAX_DT_SIMU		0.002
 #define RCM_MAX_DT_ROBOTS	0.02
+#define RCM_MAX_DT_FRAME	0.05
 
 /** General info on current race */
 typedef struct {
@@ -78,7 +79,8 @@ typedef struct {
     int			features;	/**< A list of features a race have (the common set of features all the robots in the race actually support) */
 #define RM_FEATURE_PENALTIES	0x01 /**< A robot with this feature implements penalties */
 #define RM_FEATURE_TIMEDSESSION 0x02 /**< A robot with this feature implements timed sessions */
-#define RM_FEATURE_WETTRACK	0x04 /**< A robot with this feature can handle wet track conditions */
+#define RM_FEATURE_WETTRACK	    0x04 /**< A robot with this feature can handle wet track conditions */
+#define RM_FEATURE_REALWEATHER  0x08 /**< A robot with this feature can handle with dynamic weather */
 } tRaceAdmInfo;
 
 #define _ncars		raceInfo.ncars
@@ -97,7 +99,7 @@ typedef struct Situation {
     double		currentTime;	/**< current time in sec since the beginning of the simulation */
     double		accelTime;	/**< accelerared time used for 24hr race simulation */
     int			nbPlayers;	/**< number of human player in local (splitted screen) */
-    tCarElt		**cars;		/**< list of cars */ 
+    tCarElt		**cars;		/**< list of cars */
 } tSituation;
 
 /** Race Engine states */
@@ -123,7 +125,7 @@ typedef struct Situation {
 #define RE_STATE_RACE_COOLDOWN	19
 
 /** Race Engine Car Information about the race */
-typedef struct 
+typedef struct
 {
     tTrkLocPos	prevTrkPos;
     tdble	sTime;
@@ -159,9 +161,9 @@ typedef struct
     int			running;
     unsigned	displayMode; // Bit field (see RM_DISP_MODE_*)
     tCarElt		*pitRequester; // The car asking for pit (stopped in the slot).
-	char		*message;
+    char		*message;
     double		messageEnd;
-	char		*bigMessage;
+    char		*bigMessage;
     double		bigMessageEnd;
 } tRaceEngineInfo;
 
@@ -184,9 +186,31 @@ typedef struct
 #define _reBigMessageEnd	raceEngineInfo.bigMessageEnd
 
 #define RM_PNST_DRIVETHROUGH	0x00000001
-#define RM_PNST_STOPANDGO	0x00000002
+#define RM_PNST_STOPANDGO	    0x00000002
 #define RM_PNST_STOPANDGO_OK	0x00000004
-#define RM_PNST_OVERSPEED	0x00000008
+#define RM_PNST_OVERSPEED	    0x00000008
+#define RM_PNST_SPD		        0x00010000
+#define RM_PNST_STNGO		    0x00020000
+
+typedef struct RmRaceRules
+{
+    enum RmRuleFlags
+    {
+        CORNER_CUTTING_TIME_INVALIDATE = 1,
+        WALL_HIT_TIME_INVALIDATE = 2,
+        CORNER_CUTTING_TIME_PENALTY = 4
+    };
+
+    int   enabled;
+    tdble fuelFactor;
+    tdble damageFactor;
+    tdble refuelFuelFlow;
+    tdble damageRepairFactor;
+    tdble pitstopBaseTime;
+    tdble tireFactor;
+    tdble allTiresChangeTime;
+} tRmRaceRules;
+
 
 typedef struct RmCarRules
 {
@@ -199,16 +223,17 @@ typedef struct RmCarRules
  */
 typedef struct RmInfo
 {
-    tCarElt		*carList;	/**< List of all the cars racing */
-    tSituation		*s;		/**< Situation during race */
-    tTrack		*track;		/**< Current track */
-    void		*params;	/**< Raceman parameters */
-    void		*mainParams;    /**< Stays the same even if params change because of more xml-files per raceman */
-    void		*results;	/**< Race results */
-    void		*mainResults;   /**< Stays the same even if params change because of more xml-files per raceman */
+    tCarElt		    *carList;	    /**< List of all the cars racing */
+    tSituation		*s;		        /**< Situation during race */
+    tTrack		    *track;		    /**< Current track */
+    void		    *params;	    /**< Raceman parameters */
+    void		    *mainParams;    /**< Stays the same even if params change because of more xml-files per raceman */
+    void		    *results;	    /**< Race results */
+    void		    *mainResults;   /**< Stays the same even if params change because of more xml-files per raceman */
     tModList		**robModList;	/**< robot modules loaded */
-    tRmCarRules		*rules;		/**< by car rules */
+    tRmCarRules		*rules;		    /**< by car rules */
     tRaceEngineInfo	raceEngineInfo;
+    tRmRaceRules    raceRules;
 } tRmInfo;
 
 /*
@@ -253,15 +278,15 @@ typedef struct RmInfo
 #define RM_ATTR_TOTALNUM	"total number"
 #define RM_ATTR_MAX_DRV		"maximum drivers"
 #define RM_ATTR_NUMBER		"number"
-#define RM_ATTR_CAR		"car"
+#define RM_ATTR_CAR		    "car"
 #define RM_ATTR_NBGROUPS	"number of groups"
-#define RM_ATTR_CAR_CATEGORY "Car Category"
+#define RM_ATTR_CAR_CATEGORY    "Car Category"
 
 #define RM_ATTR_PRIO		"priority"
 #define RM_ATTR_NAME		"name"
 #define RM_ATTR_TYPE		"type"
 #define RM_ATTR_SUBTYPE		"subtype"
-#define RM_ATTR_FULLNAME		"full name"
+#define RM_ATTR_FULLNAME	"full name"
 #define RM_ATTR_DRVNAME		"driver name"
 #define RM_ATTR_CATEGORY	"category"
 #define RM_ATTR_DESCR		"description"
@@ -270,8 +295,8 @@ typedef struct RmInfo
 #define RM_ATTR_STARTIMG	"start image"
 
 #define RM_ATTR_MODULE		"module"
-#define RM_ATTR_IDX		"idx"
-#define RM_ATTR_CARNAME	"car name"
+#define RM_ATTR_IDX		    "idx"
+#define RM_ATTR_CARNAME	    "car name"
 #define RM_ATTR_TEAMNAME	"team"
 #define RM_ATTR_SKINNAME	"skin name"
 #define RM_ATTR_SKINTARGETS	"skin targets"
@@ -296,11 +321,24 @@ typedef struct RmInfo
 #define RM_ATTR_SESSIONTIME	"sessiontime"
 #define RM_ATTR_CLOUDS	    "clouds"
 #define RM_ATTR_RAIN        "rain"
-#define RM_ATTR_TIME_OF_DAY		"time of day"
+#define RM_ATTR_TIME_OF_DAY	"time of day"
+#define RM_ATTR_WEATHER     "weather"
 #define RM_ATTR_QUAL_LAPS	"Qualification laps"
 #define RM_ATTR_POLE		"pole position side"
 #define RM_ATTR_CARSPERPIT	"cars per pit"
 
+#define RM_ATTR_INVALIDATE_BEST_LAP_WALL_TOUCH  "invalidate best lap on wall touch"
+#define RM_ATTR_INVALIDATE_BEST_LAP_CORNER_CUT  "invalidate best lap on corner cutting"
+#define RM_ATTR_CORNER_CUT_TIME_PENALTY         "corner cutting time penalty"
+#define RM_ATTR_DAMAGE_FACTOR                   "damage factor"
+#define RM_ATTR_FUEL_FACTOR                     "fuel consumption factor"
+#define RM_ATTR_TIRE_FACTOR                     "tire factor"
+#define RM_ATTR_PIT_SPEED_LIMIT                 "pit speed limit"
+#define RM_ATTR_REFUEL_FUEL_FLOW                "refuel fuel flow"
+#define RM_ATTR_DAMAGE_REPAIR_FACTOR            "damage repair factor"
+#define RM_ATTR_PITSTOP_BASE_TIME               "pitstop base time"
+#define RM_ATTR_ALL_TIRES_CHANGE_TIME           "all tires change time"
+
 #define RM_ATTR_POINTS		"points"
 
 #define RM_VAL_TRACKSEL		"track select"
@@ -308,17 +346,24 @@ typedef struct RmInfo
 #define RM_VAL_RACECONF		"race config"
 #define RM_VAL_ANYRACE		"All Sessions"
 
-#define RM_VAL_CONFRACELEN	"race length"
-#define RM_VAL_CONFDISPMODE	"display mode"
-#define RM_VAL_CONFTIMEOFDAY	"time of day"
-#define RM_VAL_CONFCLOUDCOVER	"cloud cover"
-#define RM_VAL_CONFRAINFALL	"rain fall"
+#ifdef CLIENT_SERVER
+#define RM_VAL_CLIENTCONF	"client config"
+#define RM_VAL_SERVERCONF	"server config"
+#endif
+
+#define RM_VAL_CONFRACELEN	   "race length"
+#define RM_VAL_CONFDISPMODE	   "display mode"
+#define RM_VAL_CONFTIMEOFDAY   "time of day"
+#define RM_VAL_CONFCLOUDCOVER  "cloud cover"
+#define RM_VAL_CONFRAINFALL	   "rain fall"
+#define RM_VAL_CONFWEATHER     "weather"
 
 #define RM_CONF_RACE_LEN	0x0001
 #define RM_CONF_DISP_MODE	0x0002
 #define RM_CONF_TIME_OF_DAY	0x0004
 #define RM_CONF_CLOUD_COVER	0x0008
 #define RM_CONF_RAIN_FALL	0x0010
+#define RM_CONF_WEATHER     0x0016
 
 #define RM_VAL_DRV_LIST_ORDER	"drivers list"
 #define RM_VAL_LAST_RACE_ORDER	"last race"
@@ -347,9 +392,9 @@ typedef struct RmInfo
 #define RM_VAL_TIME_24HR	"24 hour"
 #define RM_VAL_TIME_REAL	"real"
 #define RM_VALS_TIME { RM_VAL_TIME_DAWN, RM_VAL_TIME_MORNING, RM_VAL_TIME_NOON, \
-                       RM_VAL_TIME_AFTERNOON, RM_VAL_TIME_DUSK, RM_VAL_TIME_NIGHT, \
-                       RM_VAL_TIME_NOW, RM_VAL_TIME_TRACK, RM_VAL_TIME_RANDOM, \
-                       RM_VAL_TIME_24HR, RM_VAL_TIME_REAL}
+    RM_VAL_TIME_AFTERNOON, RM_VAL_TIME_DUSK, RM_VAL_TIME_NIGHT, \
+    RM_VAL_TIME_NOW, RM_VAL_TIME_TRACK, RM_VAL_TIME_RANDOM, \
+    RM_VAL_TIME_24HR, RM_VAL_TIME_REAL}
 
 // Indices for RM_VALS_TIME.
 #define RM_IND_TIME_DAWN	0
@@ -364,15 +409,17 @@ typedef struct RmInfo
 #define RM_IND_TIME_24HR	9
 #define RM_IND_TIME_REAL	10
 
-#define RM_VAL_CLOUDS_NONE	"no cloud"
-#define RM_VAL_CLOUDS_FEW	"few clouds"
+#define RM_VAL_CLOUDS_NONE	    "no cloud"
+#define RM_VAL_CLOUDS_CIRRUS    "cirrus"
+#define RM_VAL_CLOUDS_FEW	    "few clouds"
+#define RM_VAL_CLOUDS_MANY	    "many clouds"
+#define RM_VAL_CLOUDS_CUMULUS   "cumulus"
 #define RM_VAL_CLOUDS_SCARCE	"scarce clouds"
-#define RM_VAL_CLOUDS_MANY	"many clouds"
-#define RM_VAL_CLOUDS_FULL	"full cover"
+#define RM_VAL_CLOUDS_BROKEN    "broken"
+#define RM_VAL_CLOUDS_FULL	    "full cover"
 #define RM_VAL_CLOUDS_RANDOM	"random"
-#define RM_VAL_CLOUDS_REAL	"real"
-#define RM_VALS_CLOUDS { RM_VAL_CLOUDS_NONE, RM_VAL_CLOUDS_FEW, RM_VAL_CLOUDS_SCARCE, \
-                         RM_VAL_CLOUDS_MANY, RM_VAL_CLOUDS_FULL, RM_VAL_CLOUDS_RANDOM }
+#define RM_VAL_CLOUDS_REAL	    "real"
+#define RM_VALS_CLOUDS { RM_VAL_CLOUDS_NONE, RM_VAL_CLOUDS_CIRRUS, RM_VAL_CLOUDS_FEW, RM_VAL_CLOUDS_MANY, RM_VAL_CLOUDS_CUMULUS, RM_VAL_CLOUDS_SCARCE, RM_VAL_CLOUDS_BROKEN, RM_VAL_CLOUDS_FULL, RM_VAL_CLOUDS_RANDOM }
 
 #define RM_VAL_RAIN_NONE	"none"
 #define RM_VAL_RAIN_LITTLE	"little"
@@ -380,8 +427,12 @@ typedef struct RmInfo
 #define RM_VAL_RAIN_HEAVY	"heavy"
 #define RM_VAL_RAIN_RANDOM	"random"
 #define RM_VAL_RAIN_REAL	"real"
-#define RM_VALS_RAIN { RM_VAL_RAIN_NONE, RM_VAL_RAIN_LITTLE, RM_VAL_RAIN_MEDIUM, \
-                       RM_VAL_RAIN_HEAVY, RM_VAL_RAIN_RANDOM }
+#define RM_VALS_RAIN { RM_VAL_RAIN_NONE, RM_VAL_RAIN_LITTLE, RM_VAL_RAIN_MEDIUM, RM_VAL_RAIN_HEAVY, RM_VAL_RAIN_RANDOM }
+
+#define RM_VAL_WEATHER_CONFIG   "config"
+#define RM_VAL_WEATHER_RECORDED "recorded"
+#define RM_VAL_WEATHER_REAL     "real"
+#define RM_VALS_WEATHER { RM_VAL_WEATHER_CONFIG, RM_VAL_WEATHER_RECORDED, RM_VAL_WEATHER_REAL }
 
 /* Movie capture */
 
@@ -479,10 +530,11 @@ typedef struct RmInfo
 
 #define RE_SECT_RANK		"Rank"
 
-#define RE_SECT_TEAMINFO	"Team Info"		
+#define RE_SECT_TEAMINFO	"Team Info"
 
 #define RE_ATTR_NAME		"name"
 #define RE_ATTR_SNAME		"short name"
+#define RE_ATTR_NATION      "nation"
 #define RE_ATTR_CAR		"car"
 #define RE_ATTR_MODULE		"module"
 #define RE_ATTR_IDX		"idx"
@@ -495,9 +547,12 @@ typedef struct RmInfo
 #define RE_ATTR_DAMMAGES	"damages"
 #define RE_ATTR_NB_PIT_STOPS	"pit stops"
 #define RE_ATTR_POINTS		"points"
+#define RE_ATTR_PENALTYTIME	"penaltytime"
+
 #define RE_SECT_CLASSPOINTS	"Class Points"
 
 #endif /* _RACEMANV1_H_ */
 
 
 
+
