@@ -4,7 +4,7 @@
     created              : Sun Jan 30 22:59:40 CET 2000, 2002
     copyright            : (C) 2000 by Eric Espie
     email                : torcs@free.fr
-    version              : $Id: robot.h 5319 2013-03-15 00:22:00Z kmetykog $
+    version              : $Id: robot.h 8083 2022-03-22 20:26:22Z iobyte $
 
  ***************************************************************************/
 
@@ -16,11 +16,11 @@
  *   (at your option) any later version.                                   *
  *                                                                         *
  ***************************************************************************/
- 
-/** @file   
-    		
+
+/** @file
+
     @author	<a href=mailto:eric.espie@torcs.org>Eric Espie</a>
-    @version	$Id: robot.h 5319 2013-03-15 00:22:00Z kmetykog $
+    @version	$Id: robot.h 8083 2022-03-22 20:26:22Z iobyte $
     @ingroup	robotmodint
 */
 
@@ -32,7 +32,7 @@
    @image	html robot_call.gif
    @ingroup	modint
 */
- 
+
 #ifndef _ROBOTV1_H_
 #define _ROBOTV1_H_
 
@@ -45,6 +45,8 @@ typedef void (*tfRbNewTrack)(int index, tTrack *track, void *carHandle, void **m
 /** Callback prototype */
 typedef void (*tfRbNewRace) (int index, tCarElt *car, tSituation *s);
 /** Callback prototype */
+typedef void (*tfRbPauseRace) (int index, tCarElt *car, tSituation *s);
+/** Callback prototype */
 typedef void (*tfRbResumeRace) (int index, tCarElt *car, tSituation *s);
 /** Callback prototype */
 typedef void (*tfRbEndRace) (int index, tCarElt *car, tSituation *s);
@@ -65,14 +67,15 @@ typedef int  (*tfRbPitCmd)  (int index, tCarElt* car, tSituation *s);
 typedef struct RobotItf {
     tfRbNewTrack rbNewTrack;	/**< Give the robot the track view. Called for every track change or new race */
     tfRbNewRace  rbNewRace;	/**< Start a new race */
+    tfRbPauseRace  rbPauseRace;	/**< pause current race to ESC menu */
     tfRbResumeRace  rbResumeRace;	/**< resume current race from ESC menu */
     tfRbEndRace  rbEndRace;	/**< End of the current race */
     tfRbDrive	 rbDrive;	/**< Drive during race */
     tfRbPitCmd	 rbPitCmd;	/**< Get the driver's pit commands.
-				 <br>Returns:
-				 - ROB_PIT_IM
-				 - ROB_PIT_MENU
-				*/
+                 <br>Returns:
+                 - ROB_PIT_IM
+                 - ROB_PIT_MENU
+                */
     tfRbShutdown rbShutdown;	/**< Called before the dll is unloaded */
     int		 index;		/**< Index used if multiple interfaces */
 } tRobotItf;
@@ -82,41 +85,46 @@ typedef struct RobotItf {
 /*
  * Parameters definitions for driver
  */
-#define ROB_SECT_ROBOTS		"Robots"
+#define ROB_SECT_ROBOTS                 "Robots"
 
-#define ROB_LIST_INDEX		"index"
+#define ROB_LIST_INDEX                  "index"
 
-#define ROB_ATTR_NAME		"name"
-#define ROB_ATTR_SNAME		"short name"
-#define ROB_ATTR_CODE		"code name" //3-letter abbreviated name
-#define ROB_ATTR_TEAM		"team"
-#define ROB_ATTR_DESC		"desc"
-#define ROB_ATTR_AUTHOR		"author"
-#define ROB_ATTR_CAR		"car name"
-#define ROB_ATTR_CATEGORY	"category"
-#define ROB_ATTR_RACENUM	"race number"
-#define ROB_ATTR_RED		"red"
-#define ROB_ATTR_GREEN		"green"
-#define ROB_ATTR_BLUE		"blue"
+#define ROB_ATTR_NAME                   "name"
+#define ROB_ATTR_SNAME                  "short name"
+#define ROB_ATTR_CODE                   "code name" //3-letter abbreviated name
+#define ROB_ATTR_NATION                 "nation"    //2-letter abbreviated nationality
+#define ROB_ATTR_TEAM                   "team"
+#define ROB_ATTR_DESC                   "desc"
+#define ROB_ATTR_AUTHOR                 "author"
+#define ROB_ATTR_CAR                    "car name"
+#define ROB_ATTR_CATEGORY               "category"
+#define ROB_ATTR_RACENUM                "race number"
+#define ROB_ATTR_RED                    "red"
+#define ROB_ATTR_GREEN                  "green"
+#define ROB_ATTR_BLUE                   "blue"
 
-#define ROB_ATTR_TYPE		"type"
+#define ROB_ATTR_TYPE                   "type"
 
-#define ROB_VAL_HUMAN		"human"
-#define ROB_VAL_ROBOT		"robot"
-#define ROB_ATTR_LEVEL		"skill level"
-#define ROB_ATTR_FEATURES	"features"
+#define ROB_VAL_HUMAN                   "human"
+#define ROB_VAL_ROBOT                   "robot"
+#define ROB_ATTR_LEVEL                  "skill level"
+#define ROB_ATTR_FEATURES               "features"
 
-#define ROB_VAL_ROOKIE		"rookie"
-#define ROB_VAL_AMATEUR		"amateur"
-#define ROB_VAL_SEMI_PRO	"semi-pro"
-#define ROB_VAL_PRO		"pro"
-#define ROB_SECT_ARBITRARY	"Robots/arbitrary cars"
+#define ROB_VAL_ARCADE					"arcade"
+#define ROB_VAL_SEMI_ROOKIE				"semi-rookie"
+#define ROB_VAL_ROOKIE                  "rookie"
+#define ROB_VAL_AMATEUR                 "amateur"
+#define ROB_VAL_SEMI_PRO                "semi-pro"
+#define ROB_VAL_PRO                     "pro"
+#define ROB_VALS_LEVEL { ROB_VAL_ARCADE, ROB_VAL_SEMI_ROOKIE, ROB_VAL_ROOKIE, ROB_VAL_AMATEUR, ROB_VAL_SEMI_PRO, ROB_VAL_PRO }
+#define ROB_SECT_ARBITRARY              "Robots/arbitrary cars"
 
-#define ROB_VAL_FEATURE_PENALTIES	"penalties"
+#define ROB_VAL_FEATURE_PENALTIES       "penalties"
 #define ROB_VAL_FEATURE_TIMEDSESSION	"timed session"
 #define ROB_VAL_FEATURE_WETTRACK		"wet track"
+#define ROB_VAL_FEATURE_REALWEATHER     "real weather"
 
-#endif /* _ROBOTV1_H_ */ 
+#endif /* _ROBOTV1_H_ */
 
 
 
