@@ -4,7 +4,7 @@
     created              : Fri Aug 13 22:32:14 CEST 1999
     copyright            : (C) 1999 by Eric Espie
     email                : torcs@free.fr
-    version              : $Id: tgf.h 6270 2015-11-23 19:44:40Z madbad $
+    version              : $Id: tgf.h 8709 2023-01-29 05:22:23Z iobyte $
  ***************************************************************************/
 
 /***************************************************************************
@@ -19,7 +19,7 @@
 /** @file   
     	The Gaming Framework API.
     @author	<a href=mailto:torcs@free.fr>Eric Espie</a>
-    @version	$Id: tgf.h 6270 2015-11-23 19:44:40Z madbad $
+    @version	$Id: tgf.h 8709 2023-01-29 05:22:23Z iobyte $
 */
 
 #ifndef __TGF__H__
@@ -46,6 +46,7 @@
 #include "modinfo.h" // Don't move this include line : needs TGF_API definition.
 
 #include <string>
+#include <vector>
 
 /** Floating point type used everywhere.
     @ingroup definitions
@@ -78,6 +79,7 @@ const double PI_3 = PI/3; /**< PI/3 =  60 deg */
 const double PI_4 = PI/4; /**< PI/4 =  45 deg */
 const double PI_6 = PI/6; /**< PI/6 =  30 deg */
 const tdble G = 9.80665f; /**< m/s/s */
+const double ZEROC = 273.15; /**< 0 degrees Celsius in Kelvin */
 
 /* conversion */
 #define RADS2RPM(x) ((x)*9.549296585)		/**< Radian/s to RPM conversion */
@@ -88,6 +90,8 @@ const tdble G = 9.80665f; /**< m/s/s */
 #define FLOAT_DEG2RAD(x)  ((x)*(float)(PI/180.0))/**< Degree to radian conversion */
 #define FEET2M(x)   ((x)*0.304801)		/**< Feet to meter conversion */
 #define SIGN(x)     ((x) < 0 ? -1.0 : 1.0)	/**< Sign of the expression */
+#define C2K(x)      ((x) + ZEROC)		/**< Celsius to Kelvin conversion */
+#define K2C(x)      ((x) - ZEROC)		/**< Kelvin to Celsius conversion */
 
 /** Angle normalization between 0 and 2 * PI */
 #define NORM0_2PI(x) 				\
@@ -181,6 +185,7 @@ typedef struct Forces
  ******************************/
 TGF_API void GfInit(bool bWithLogging = true);
 TGF_API void GfShutdown(void);
+TGF_API void GfTraceShutdown(void);
 
 
 /************************************************************************
@@ -372,6 +377,7 @@ TGF_API bool GfFileCopy(const char* pszSrcName, const char* pszTgtName);
 TGF_API bool GfPathIsAbsolute(const char *pszPath);
 TGF_API char* GfPathNormalizeDir(char* pszPath, size_t nMaxPathLen);
 TGF_API char* GfPathMakeOSCompatible(char* path);
+TGF_API char* GfPathNormalizeFile(char* pszPath, size_t nMaxPathLen);
 
 
 /**********************************
@@ -401,21 +407,23 @@ TGF_API char* GfPathMakeOSCompatible(char* path);
 
 /* parameter file read */
 TGF_API void *GfParmReadFileLocal(const char *file, int mode, bool neededFile = true);
+TGF_API void *GfParmReadFileLocal(const std::string &file, int mode, bool neededFile = true);
 /* last optional parameter allows usage without logger be available */
 TGF_API void *GfParmReadFile(const char *file, int mode, bool neededFile = true, bool trace = true);
+TGF_API void *GfParmReadFile(const std::string &file, int mode, bool neededFile = true, bool trace = true);
 
 TGF_API void *GfParmReadBuf(char *buffer);
 
 /* parameter file write */
 TGF_API int GfParmWriteFileLocal(const char *file, void* handle, const char *name);
+TGF_API int GfParmWriteFileLocal(const std::string &file, void *handle, const char *name);
 TGF_API int GfParmWriteFile(const char *file, void* handle, const char *name);
+TGF_API int GfParmWriteFile(const std::string &file, void *handle, const char *name);
 /* last optional parameter allows usage without logger be available */
 TGF_API int GfParmWriteFileSDHeader(const char *file, void* handle, const char *name, const char *author, bool trace = true);
 /* write a parameter file to a supplied buffer */
 TGF_API int GfParmWriteBuf(void *handle, char *buf, int size);/* Never used in current codebase: to be removed? */
-#ifdef WEBSERVER
 TGF_API int GfParmWriteString(void *handle, std::string& str);
-#endif //WEBSERVER
 
 TGF_API char *GfParmGetName(void *handle);
 TGF_API char *GfParmGetFileName(void *handle);
@@ -428,11 +436,14 @@ TGF_API void GfParmSetDTD (void *parmHandle, char *dtd, char*header);
 /* get string parameter value */
 TGF_API const char *GfParmGetStr(void *handle, const char *path, const char *key, const char *deflt);
 TGF_API char *GfParmGetStrNC(void *handle, const char *path, const char *key, char *deflt);
+TGF_API std::vector<std::string> GfParmGetStrIn(void *handle, const char *path, const char *key);
 /* get string parameter value */
 TGF_API const char *GfParmGetCurStr(void *handle, const char *path, const char *key, const char *deflt);
 TGF_API char *GfParmGetCurStrNC(void *handle, const char *path, const char *key, char *deflt);
 /* set string parameter value */
 TGF_API int GfParmSetStr(void *handle, const char *path, const char *key, const char *val);
+TGF_API int GfParmSetStrIn(void *handle, const char *path, const char *key, const std::vector<std::string> &in);
+TGF_API int GfParmSetStrAndIn(void *handle, const char *path, const char *key, const char *val, const std::vector<std::string> &in);
 /* set string parameter value */
 TGF_API int GfParmSetCurStr(void *handle, const char *path, const char *key, const char *val);
 
@@ -488,7 +499,9 @@ TGF_API int GfParmGetNumBoundaries(void *handle, char *path, char *key, tdble *m
 
 TGF_API void GfParmRemove(void *handle, const char *path, const char *key);
 TGF_API int GfParmRemoveSection (void *handle, const char *path);
-TGF_API int GfParmExistsSection(void *handle, const char *path);
+TGF_API bool GfParmExistsSection(void *handle, const char *path);
+TGF_API  std::vector<std::string> GfParmListGetSectionNamesList(void *handle);
+TGF_API bool GfParmExistsParam(void *handle, const char *path, const char *key);
 TGF_API int GfParmGetEltNb(void *handle, const char *path);
 TGF_API int GfParmListSeekFirst(void *handle, const char *path);
 TGF_API int GfParmListSeekNext(void *handle, const char *path);
@@ -496,6 +509,8 @@ TGF_API char *GfParmListGetCurEltName(void *handle, const char *path);
 TGF_API int GfParmListRemoveElt(void *handle, const char *path, const char *key);
 TGF_API int GfParmListRenameElt(void *handle, const char *path, const char *oldKey, const char *newKey);
 TGF_API int GfParmListClean(void *handle, const char *path);
+TGF_API  std::vector<std::string> GfParmListGetParamsNamesList(void *handle, const char *path);
+
 
 TGF_API void GfParmSetVariable(void *handle, char const *path, char const *key, tdble val);
 TGF_API void GfParmRemoveVariable(void *handle, char const *path, char const *key);
@@ -551,6 +566,19 @@ TGF_API void GfParmShutdown (void);
  *    <user settings>/config/logging.xml (see source in data/config)            *
  ********************************************************************************/
 
+// Tell the compiler a function is using a printf-style format string.
+// |format_param| is the one-based index of the format string parameter;
+// |dots_param| is the one-based index of the "..." parameter.
+// For v*printf functions (which take a va_list), pass 0 for dots_param.
+// (This is undocumented but matches what the system C headers do.)
+// For member functions, the implicit this parameter counts as index 1.
+#if defined(__GNUC__)
+#define PRINTF_FORMAT(format_param, dots_param) \
+  __attribute__((format(printf, format_param, dots_param)))
+#else
+#define PRINTF_FORMAT(format_param, dots_param)
+#endif
+
 //****************************************
 // Logger class
 
@@ -584,11 +612,11 @@ class TGF_API GfLogger
 	//! Tracing functions (name gives the trace level / criticity).
 	void fatal(const char *pszFmt, ...); // Warning : This one calls exit(1) at the end !
 #ifdef TRACE_OUT
-	void error(const char *pszFmt, ...);
-	void warning(const char *pszFmt, ...);
-	void info(const char *pszFmt, ...);
-	void trace(const char *pszFmt, ...);
-	void debug(const char *pszFmt, ...);
+	void error(const char *pszFmt, ...) PRINTF_FORMAT(2, 3);
+	void warning(const char *pszFmt, ...) PRINTF_FORMAT(2, 3);
+	void info(const char *pszFmt, ...) PRINTF_FORMAT(2, 3);
+	void trace(const char *pszFmt, ...) PRINTF_FORMAT(2, 3);
+	void debug(const char *pszFmt, ...) PRINTF_FORMAT(2, 3);
 #else // TRACE_OUT
 	// The compiler should simply skip calls to these ...
 	inline void error(const char *pszFmt, ...) {};
@@ -600,7 +628,7 @@ class TGF_API GfLogger
 
 	//! Generic tracing function (you must specify the level, enum or integer if > eDebug).
 #ifdef TRACE_OUT
-	void message(int nLevel, const char *pszFmt, ...);
+	void message(int nLevel, const char *pszFmt, ...) PRINTF_FORMAT(3, 4);
 #else // TRACE_OUT
 	// The compiler should simply skip calls to this ...
 	inline void message(int nLevel, const char *pszFmt, ...) {};
@@ -997,7 +1025,6 @@ TGF_API void GfFormFreeCommandNew(void *cmd);
 // Maximun value of a key code (Has to be the least greater  2^N - 1 >= SDLK_LAST)
 #define GF_MAX_KEYCODE	0x1FF
 
-
 #endif /* __TGF__H__ */
 
 
