@@ -4,7 +4,7 @@
     created              : Mod Mar 14 20:32:14 CEST 2011
     copyright            : (C) 2011 by Jean-Philippe Meuret
     web                  : http://www.speed-dreams.org
-    version              : $Id: tgf.hpp 5366 2013-03-26 19:39:49Z pouillot $
+    version              : $Id: tgf.hpp 8058 2022-02-28 04:15:39Z beaglejoe $
 ***************************************************************************/
 
 /***************************************************************************
@@ -18,7 +18,7 @@
 
 /** @file   
     	The Gaming Framework API, with C++-specific extensions.
-    @version	$Id: tgf.hpp 5366 2013-03-26 19:39:49Z pouillot $
+    @version	$Id: tgf.hpp 8058 2022-02-28 04:15:39Z beaglejoe $
 */
 
 #ifndef __TGF__HPP__
@@ -207,7 +207,7 @@ class TGF_API GfApplication
 	void addOptionsHelpExplainLine(const std::string& strTextLine);
 	
 	//! Parse the command line for registered options.
-	bool parseOptions();
+	virtual bool parseOptions();
 
 	//! Check if we have the specified regsitered option in the command line.
 	bool hasOption(const std::string& strLongName) const; // No leading '--'
@@ -238,6 +238,11 @@ class TGF_API GfApplication
 	//! Restart the app.
 	virtual void restart();
 
+	// Allow clean restart
+#ifdef __DEBUG_MEMORYMANAGER__
+	void (*ReleaseData)();
+#endif
+
  protected:
 
 	//! Print a short help about using the command line.
@@ -273,7 +278,7 @@ class TGF_API GfApplication
 		bool bFound;
 		std::string strValue;
 	public:
-		Option()
+		Option() : bHasValue(false), bFound(false)
 		{
 		}
 		Option(const std::string& strShortName_, const std::string& strLongName_,
